#include <Servo.h>
#include <Arduino.h>
#pragma once

/* Topfempfaenger ist ein Modul bestehend aus einem Infrarotsensor und 
 einem Servo, der ein Signal erfassen und die Richtung ermitteln soll,
 aus dem es kommt.
 */
class Topfempfaenger {
private:  
  Servo myServo; 
  int recPin; // Pin des Infrarotsensors
  int servPin; // Pin des Servos
  int positionS; // aktuelle position des Servos (0 - 180)

  /* peileAn dreht den Servo/Sensor um 180° gegen den Uhrzeigersinn von rechts nach links
   und zurueck. Dabei ueberprueft er für jeden Schritt, ob ein Signal erfasst wurde.
   Ist dies der Fall stoppt er.  Wurde kein Signal gefunden, wird positionS auf 999 gesetzt
   und kann spaeter als "kein Signal" interpretiert werden.
   */
  void peileAn(){
    // position des Servos
    for(positionS=0; positionS<=180; positionS++){
      myServo.write(positionS);
      if(signalda(0)){
        return;
      }
    }
    // Dreh dich von der zuletzt empfangenen Position von rechts nach links zurück
    for (positionS; positionS>0; positionS--){
      myServo.write(positionS);
      if(signalda(0)){
        return;
      }
    }
    positionS = 999;
  }

public:
  void setup(int newRecPin, int newServPin){ 
    recPin = newRecPin;
    servPin = newServPin;
    pinMode(recPin,INPUT);
    pinMode(servPin, OUTPUT);
    myServo.attach(servPin); 
    positionS=0; // 0 als Startposition, 90° Abweichung zur Laengsachse
    myServo.write(positionS);
  }

  /*
searchAngle() ob an der aktuellen Position ein Signal vorhanden ist.
   Falls nicht, wird nach einem Signal gesucht und der Winkel, in dem
   es gefunden wurde, in rad umgerechnet und ubergeben.
   Der Delay ist eingebaut, damit nicht schon waehrend dem Drehen in die
   Ausgangsposition ein Signal empfangen wird und damit die Signalrichtung
   falsch zugeordnet wird.
   */
  float searchAngle(){
    if(!signalda(1)) {
      myServo.write(0);
      delay(1000);
      peileAn();
    }
    if(positionS == 999){
      return positionS;
    }
    float angle = positionS - 90;
    //float angle_rad = map(angle,-90,90,-PI/2,PI/2); //Winkel in rad
    return angle*PI/180;
  }

  /*
signalda ueberprueft, ob bei 500 Messungen ein Signal empfangen wurde.
   Da der Infrarotsender hochfrequent zwischen an und aus umschaltet, koennten
   bei einer zu hohen Programmgeschwindigkeit und Interpretation der Daten direkt nach einer
   Sensorauswertung falsche Schluesse gezogen werden (kein Signal trotz Signal).
   Fuer das Geradeausfahren sind 500 Messungen veranlagt. Beim Suchlauf im Ruhezustand
   nur die Haelfte, um den Prozess so kurz wie moeglich zu halten.
   */
  boolean signalda(int mode) {
    int meas = 120; 
    if(mode == 1) {
      meas = 500;
    }
    for(int i = 0; i < meas; i++) {
      if(analogRead(recPin) == LOW) {
        return true;
      }
    }
    return  false;
  }
};



